@@ -50,6 +50,7 @@ globalAllTemplatesProcessed = {}
 baseJ2Env = jinja2.Environment(trim_blocks=True, lstrip_blocks=True, keep_trailing_newline=True)
 CombineLists = False
 args = None
+workingValsPointer = None
 
 CurrentLoggingSettings = LoggingSettings()
 
@@ -216,6 +217,9 @@ def LoadCustomFilters():
     baseJ2Env.filters.update({f[0]: f[1]})
   for f in inspect.getmembers(global_extensions, inspect.isfunction):
     baseJ2Env.globals.update({f[0]: f[1]})
+  baseJ2Env.filters.update({
+    'nested_template': NestedTemplate
+  })
 
 ##########################################
 # Logging
@@ -247,6 +251,30 @@ def logToFile(path:str, msg:str):
     with open(path, 'a+') as f:
       f.write(msg + "\n")
 
+def NestedTemplate(filename):
+  if os.path.exists(filename):
+    dest = filename
+  elif os.path.exists(os.path.abspath(filename)):
+    dest = os.path.abspath(filename)
+  else:
+    raise Exception(f"jinny.filter_extenions.nested_template(): The file at path {filename} does not exist!")
+
+  Log(f'NestedTemplate(): {dest}')
+  nt = TemplateHandler(
+    templateName=f'(Nested) - {os.path.basename(dest)}',
+    addToGlobal=False,
+    nested=True,
+    path=dest,
+  )
+  try:
+    res = nt.Render(workingValsPointer)
+    return nt.Result()
+  except Exception as e:
+    print(f"jinny.filter_extenions.nested_template(): Failed to read and template from file {filename} with exception")
+    traceback.print_exception(e)
+    raise
+
+
 ##########################################
 # Argparsing
 def ArgParsing():
@@ -286,7 +314,7 @@ You can modify jinja's environment settings via the rest of the command line opt
 
   # Core arguments
   parser.add_argument("-v", "--verbose", help="Set output to verbose", action="store_true")
-  parser.add_argument("-vvv", "--super-verbose", help="Set output to super verbose where this script will print basically everything", action="store_true")
+  parser.add_argument("-vvv", "--super-verbose", help="Set output to super verbose where this script will print basically everything, INCLUDING POTENTIALLY SENSITIVE THINGS!", action="store_true")
   parser.add_argument("-i", "--inputs", help="Add one or more file locations that include input values to the templating", action="append", nargs="*")
   parser.add_argument("-e", "--explicit", help="Explicitly define a variable that trumps all other variables using a variable=value format. Adding variables like this trumps every other setting for that variable", action="append", nargs="*")
   parser.add_argument("-t", "--templates", help="Add one or more file locations that contain the templates themselves", action="append", nargs="*", required=True)
@@ -308,6 +336,7 @@ You can modify jinja's environment settings via the rest of the command line opt
 
   # Other Arguments
   parser.add_argument("-d", "--dump-to-dir", help="Dump completed templates to a target directory", type=str)
+  parser.add_argument("-di", "--dump-to-dir-no-index", help="Dump completed templates to a target directory without index separation, meaning that templates with the same name can overwrite prior templates", type=str)
   parser.add_argument("-s", "--stdout-seperator", help="Place a seperator on it's own individual new line between successfully templated template when printing to stdout, eg '---' for yaml", type=str, default='')
   parser.add_argument("-c", "--combine-lists", help="When cascading values across multiple files and encountering two lists with the same key, choose to combine the old list with the new list rather than have the new list replace the old", action="store_false")
   parser.add_argument("-ld", "--log-destination", help="Chose an alternate destination to log to, jinny defaults to stdout but you can provide a file to print output to instead", default="/dev/stdout", type=str)
@@ -335,13 +364,14 @@ You can modify jinja's environment settings via the rest of the command line opt
   if args.log_destination != "/dev/stdout" and not os.path.isdir(os.path.dirname(args.log_destination)):
     Log(f"ArgParsing(): The parent directory for the custom log file '{args.log_destination}' either does not exist or is not a directory, please check it!", quitWithStatus=1)
 
-  if type(args.dump_to_dir) == str:
-    args.dump_to_dir = os.path.abspath(args.dump_to_dir.strip('/'))
-    if not os.path.isdir(args.dump_to_dir):
-      Log(f"ArgParsing(): The parent directory for dumping completed templates to '{args.dump_to_dir}' does not exist, please check your arguments!", quitWithStatus=1)
+  if type(args.dump_to_dir) == str or type(args.dump_to_dir_no_index) == str:
+    tarDump = "dump_to_dir" if type(args.dump_to_dir) == str else "dump_to_dir_no_index"
+    args.__setattr__(tarDump, getattr(args, tarDump).strip('/'))
+    if not os.path.isdir(getattr(args, tarDump)):
+      Log(f"ArgParsing(): The parent directory for dumping completed templates to '{getattr(args, tarDump)}' does not exist, please check your arguments!", quitWithStatus=1)
 
-    if os.path.isfile(args.dump_to_dir):
-      Log(f"ArgParsing(): The location provided for dumping completed templates to is a file and not a directory '{args.dump_to_dir}', please check your arguments!", quitWithStatus=1)
+    if os.path.isfile(getattr(args, tarDump)):
+      Log(f"ArgParsing(): The location provided for dumping completed templates to is a file and not a directory '{getattr(args, tarDump)}', please check your arguments!", quitWithStatus=1)
 
   vs = 0
   if args.verbose:
@@ -355,18 +385,20 @@ You can modify jinja's environment settings via the rest of the command line opt
 ##########################################
 # Class
 class TemplateHandler():
-  def __init__(self, templateName:str="", addToGlobal:bool=False, path:str="", rawString:str=""):
+  def __init__(self, templateName:str="", addToGlobal:bool=False, path:str="", rawString:str="", nested:bool=False):
+    Log(f'TemplateHandler(): Handling template at path {path}')
     if not path and not rawString:
       raise Exception(f'TemplateHandler(): Neither a path nor a raw template string was provided, crashing!')
     if path and rawString:
       raise Exception(f'TemplateHandler(): Both a path and a raw string was provided, exiting, wise up!')
     self.path = path
     self.result = None
+    self.nested = nested
 
     if path != "":
       self.name = os.path.abspath(path)
       if not os.path.exists(path):
-        Log(f"TemplateHandler(): Failed to read template at path '{path}', cannot load in desired template!", quitWithStatus=1)
+        Log(f"TemplateHandler(): Failed to read {'nested template' if self.nested else 'template' } at path '{path}', cannot load in desired template!", quitWithStatus=1)
       with open(path, "r") as f:
         self.templateData = f.read()
         self.basename = os.path.basename(self.name)
@@ -381,25 +413,34 @@ class TemplateHandler():
       self.loadedTemplate = baseJ2Env.from_string(self.templateData)
     except Exception as e:
       execDetails = sys.exc_info()
-      Log(f"TemplateHandler(): Failed to load template at '{path}' with an exception from Jinja, details:\nType:{execDetails[0]}\nValue:{execDetails[1]}\nTrace:\n{traceback.format_exc()}", quitWithStatus=1)
+      Log(f"TemplateHandler(): Failed to load {'nested template' if self.nested  else 'template' } at '{path}' with an exception from Jinja, details:\nType:{execDetails[0]}\nValue:{execDetails[1]}\nTrace:\n{traceback.format_exc()}", quitWithStatus=1)
 
-    self.loadedTemplate.environment.globals["path"] = {
-      "cwd": os.getcwd().rstrip("/") + "/",
-      "jinny": baseDir.rstrip("/") + "/",
-      "template": os.path.abspath(path) if path else "",
-      "templatedir": os.path.dirname(os.path.abspath(path)).rstrip("/") + "/" if path else "",
-      "home": os.path.expanduser('~').rstrip("/") + "/"
+    self.extensions = {
+      "path": {
+        "cwd": os.getcwd().rstrip("/") + "/",
+        "jinny": baseDir.rstrip("/") + "/",
+        "template": os.path.abspath(path) if path else "",
+        "templatedir": os.path.dirname(os.path.abspath(path)).rstrip("/") + "/" if path else "",
+        "home": os.path.expanduser('~').rstrip("/") + "/"
+      }
     }
 
     if addToGlobal:
       globalAllTemplatesProcessed[self.name] = self
 
   def Render(self, values):
+    Log(f"TemplateHandler.Render({self.name}): Started render")
+    self.loadedTemplate.environment.globals["path"] = self.extensions["path"]
     try:
+      if not self.nested:
+        global workingValsPointer
+        workingValsPointer = values
       self.result = self.loadedTemplate.render(values)
+      if not self.nested:
+        workingValsPointer = None
     except Exception as e:
       execDetails = sys.exc_info()
-      Log(f"TemplateHandler.Render(): Failed to render template at '{self.path}' with an exception from Jinja, details:\nType:{execDetails[0]}\nValue:{execDetails[1]}\nTrace:\n{traceback.format_exc()}", quitWithStatus=1)
+      Log(f"TemplateHandler.Render(): Failed to render {'nested template' if self.nested  else 'template' } at '{self.path}' with an exception from Jinja, details:\nType:{execDetails[0]}\nValue:{execDetails[1]}\nTrace:\n{traceback.format_exc()}", quitWithStatus=1)
 
   def Result(self):
     return self.result if self.result else None
@@ -492,20 +533,24 @@ def Main():
 
   ##########################################
   # Templating
+  if args.super_verbose:
+    Log("Calculated values:")
+    print(json.dumps(overallValues, indent=2))
   for ind, tmpl in enumerate(globalAllTemplatesProcessed):
     globalAllTemplatesProcessed[tmpl].Render(overallValues)
 
     if globalAllTemplatesProcessed[tmpl].result == "":
       continue
 
-    if args.dump_to_dir:
-      dest = args.dump_to_dir + f'/{ind}-{globalAllTemplatesProcessed[tmpl].basename}'
+    if args.dump_to_dir or args.dump_to_dir_no_index:
+      tarDump = "dump_to_dir" if type(args.dump_to_dir) == str else "dump_to_dir_no_index"
+      dest = getattr(args, tarDump) + f'/{str(ind) + "-" if args.dump_to_dir else "" }{globalAllTemplatesProcessed[tmpl].basename}'
       with open(dest, "w+") as f:
         f.write(globalAllTemplatesProcessed[tmpl].result)
     else:
       stdoutDump.append(globalAllTemplatesProcessed[tmpl].result)
 
-  if not args.dump_to_dir:
+  if not args.dump_to_dir and not args.dump_to_dir_no_index:
     print(f'\n{args.stdout_seperator}\n'.join(stdoutDump))
 
 if __name__ == "__main__":
